/*
  Copyright 2005 Damien Stehl.

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along
  with this program; see the file COPYING.  If not, write to the Free
  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <float.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <ctype.h>
#include <limits.h>
#include <gmp.h>

#ifdef FORCE_LONGDOUBLE
#define double long double
#endif

#include "basicfast.h"


double*
init_vecf (int n)
{
  double *a;
  a = (double*) malloc (n * sizeof (double));
  return a;
}


void
init_matrixf (double **B, int d, int n)
{
  int i;

  B = (double **) malloc ((d + 1) * sizeof (double*));
  for (i=0; i<=d; i++)
    B[i] = (double*) malloc ((n+1) * sizeof (double));
}



void
clear_matrixf (double **B, int d)
{
  int i;
  for (i=0; i<=d; i++)
    free (B[i]);
  free (B);
}

void
Print_matf (double **B, int *expo, int d, int n) 
{
  int i, j; 

  printf("[");
  for (i=1;i<=d;i++) {
    printf("[");
    for (j=1;j<=n;j++) { 
#ifdef FORCE_LONGDOUBLE
      printf("%1.5Le", B[i][j]); 
#else
      printf("%E", B[i][j]); 
#endif
      if (j < n) printf(" "); 
    }
    printf("] * 2^%d\n", expo[i]); 
  }  
  printf("]\n"); 
}





int 
set_line (double *appv, mpz_t *v, int n)
{
  signed long int *exp, i, maxexp = 0;
  exp = (signed long int *) malloc((n+1) * sizeof(signed long int)); 

  for (i=1; i<=n; i++)
    {
      appv[i] = mpz_get_d_2exp (&exp[i], v[i]);
      if (exp[i]>maxexp) maxexp=exp[i];
    }

  for (i=1; i<=n; i++)
    appv[i] = ldexp( appv[i], exp[i]-maxexp);

  free(exp);
  return maxexp;
}


