/*
  Copyright 2005 Damien Stehl.

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along
  with this program; see the file COPYING.  If not, write to the Free
  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.
*/


#include <gmp.h>

#include "fpclass.h"

#ifndef NOT_USING_MPFR
#define USING_MPFR
#endif

#ifdef USING_MPFR
#include <mpfr.h>
#endif


#ifdef USE_DOUBLE

mp_exp_t
double_get_z_exp (mpz_t x, double *y)
{
  mpfr_t tmp;
  mp_exp_t resul;
  mpfr_init2 (tmp, 53);

  mpfr_set_d (tmp, y[0], GMP_RNDN);
  resul = mpfr_get_z_exp (x, tmp);
  mpfr_clear (tmp);

  return resul;
}

int
double_get_z (mpz_t a, double *b)
{
  mpfr_t tmp;
  mpfr_init2 (tmp, 53);

  mpfr_set_d (tmp, b[0], GMP_RNDN);
  mpfr_get_z (a, tmp, GMP_RNDN);
  mpfr_clear (tmp);
  return 0;
}

#else
#ifdef USE_LONGDOUBLE

#ifdef USING_MPFR
void
long_double_set_z (long double *a, mpz_t b)
{
  mpfr_t tmp;
  mpfr_init2 (tmp, 53);
  mpfr_set_z (tmp, b, GMP_RNDN);
  a[0] = mpfr_get_ld (tmp, GMP_RNDN);
  mpfr_clear(tmp);
}

mp_exp_t
long_double_get_z_exp (mpz_t x, long double *y)
{
  mpfr_t tmp;
  mp_exp_t resul;
  mpfr_init2 (tmp, 53);

  mpfr_set_ld (tmp, y[0], GMP_RNDN);
  resul = mpfr_get_z_exp (x, tmp);  
  mpfr_clear (tmp);

  return resul;
}

int
long_double_get_z (mpz_t a, long double *b)
{
  mpfr_t tmp;
  mpfr_init2 (tmp, 53);

  mpfr_set_ld (tmp, b[0], GMP_RNDN);
  mpfr_get_z (a, tmp, GMP_RNDN);
  mpfr_clear (tmp);
  return 0;
}
#endif

#endif
#endif
