/*
  Copyright 2005 Damien Stehl.

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along
  with this program; see the file COPYING.  If not, write to the Free
  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.
*/

/* Choose your floating-point numbers class */

// #define USE_MPFR
// #define USE_DOUBLE
// #define USE_DPE

#ifdef USE_MPFR

#define FP_NR mpfr_t
#define NO_DOUBLE

#define INIT(a) mpfr_init(a) 
#define CLEAR(a) mpfr_clear(a)
#define OUT(a) mpfr_out_str(stdout, 10, 10, a, GMP_RNDN) 
#define OUT2(a) mpfr_out_str(stderr, 10, 5, a, GMP_RNDN) 
#define INP_STR(a) mpfr_inp_str (a, stdin, 10, GMP_RNDN)
#define SET(a,b) mpfr_set (a,b, GMP_RNDN)
#define SET_D(a,b) mpfr_set_d (a, b, GMP_RNDN)
#define GET_D(b) mpfr_get_d (b, GMP_RNDN)
#define SET_UI(a,b) mpfr_set_ui (a, b, GMP_RNDN)
#define SET_Z(a,b) mpfr_set_z (a, b, GMP_RNDN)
#define GET_Z(a,b) mpfr_get_z (a, b, GMP_RNDN)
#define GET_SI(a,b) a = mpfr_get_si (b, GMP_RNDN)
#define EXP(a) mpfr_get_exp(a)
#define PREC(a) mpfr_get_prec(a)

#define MUL(a,b,c) mpfr_mul(a,b,c, GMP_RNDN)
#define ADD(a,b,c) mpfr_add(a,b,c, GMP_RNDN)
#define SUB(a,b,c) mpfr_sub(a,b,c, GMP_RNDN)
#define DIV(a,b,c) mpfr_div(a,b,c, GMP_RNDN) 
#define DIV_2UI(a,b,c) mpfr_div_2ui(a,b,c, GMP_RNDN)
#define MUL_2UI(a,b,c) mpfr_mul_2ui (a,b,c, GMP_RNDN)

#define ABS(a,b) mpfr_abs (a,b, GMP_RNDN)
#define CMP(a,b) mpfr_cmp (a,b)
#define CMP_D(a,b) mpfr_cmp_d (a,b)
#define SGN(a) mpfr_sgn (a)
#define RND(a,b) mpfr_round (a,b)
#define GET_Z_EXP(a,b) mpfr_get_z_exp(a,b) 
#define SWAP(a,b) mpfr_swap (a,b)
#define ZERO_P(a) mpfr_zero_p (a)

#define SET_NAN(a) mpfr_set_nan (a) 
#define IS_NAN(a) mpfr_nan_p (a)
#define SQRT(a,b) mpfr_sqrt (a, b, GMP_RNDN)


#else
#ifdef USE_DOUBLE

typedef double double_t[1];

#define FP_NR double_t
#define INIT(a)  {}
#define CLEAR(a) {}
#define OUT(a) printf ("%E ", a[0]) 
#define OUT2(a) fprintf (stderr, "%E ", a[0]) 
#define SET(a,b) a[0] = b[0]
#define SET_D(a,b) a[0] = b
#define GET_D(b) b[0]
#define SET_UI(a,b) a[0] = (double) b
#define SET_Z(a,b) a[0]= mpz_get_d (b)

#define MUL(a,b,c) a[0] = b[0] * c[0]
#define ADD(a,b,c) a[0] = b[0] + c[0]
#define SUB(a,b,c) a[0] = b[0] - c[0]
#define DIV(a,b,c) a[0] = b[0] / c[0] 
#define DIV_2UI(a,b,c) a[0]= ldexp (b[0], -c)
#define MUL_2UI(a,b,c) a[0]= ldexp (b[0], c)

#define ABS(a,b) a[0] = (b[0]>=0.0? b[0]: -b[0])
#define CMP(a,b) a[0] <= b[0] ? -1 : 1
#define CMP_D(a,b) a[0] <= b ? -1 : 1
#define SGN(a) (a[0]>0.0? 1: (a[0]==0.0? 0 : -1))
#define RND(a,b) a[0] = rint(b[0])
#define ZERO_P(a) (a==0)

mp_exp_t double_get_z_exp (mpz_t, double*);
int double_get_z (mpz_t, double*);

#define GET_SI(a,b) a = (signed long int) b[0]
#define GET_Z_EXP(a,b) double_get_z_exp (a,b)
#define GET_Z(a,b) double_get_z (a,b)

#define SET_NAN(a) a[0] = 0.0/0.0
#define IS_NAN(a) (a[0]!=a[0])
#define SQRT(a,b) a[0] = sqrt(b[0])

#else
#ifdef USE_DPE
#define DPE_USE_DOUBLE
// #define inline 

#include "dpe/dpe.h"
#define FP_NR dpe_t
#define NO_DOUBLE
#define EXP(a) DPE_EXP(a)
#define PREC(a) 53
#define INIT(a) dpe_init(a)
#define CLEAR(a) dpe_clear(a)
#define OUT(a) dpe_out_str(stdout, 10, a) 
#define OUT2(a) dpe_out_str(stderr, 10, a) 
#define INP_STR(a) dpe_inp_str (a, stdin, 10)
#define SET(a,b) dpe_set (a,b)
#define SET_D(a,b) dpe_set_d (a,b)
#define GET_D(b) dpe_get_d (b)
#define SET_UI(a,b) dpe_set_d (a, (double) b)
#define SET_Z(a,b) dpe_set_z (a,b)
#define GET_Z(a,b) dpe_get_z (a,b)
#define GET_SI(a,b) a = dpe_get_si (b)

#define MUL(a,b,c) dpe_mul(a,b,c)
#define ADD(a,b,c) dpe_add(a,b,c)
#define SUB(a,b,c) dpe_sub(a,b,c)
#define DIV(a,b,c) dpe_div(a,b,c) 
#define DIV_2UI(a,b,c) dpe_div_2exp(a,b,c)
#define MUL_2UI(a,b,c) dpe_mul_2exp(a,b,c)

#define ABS(a,b) dpe_abs (a,b)
#define CMP(a,b) dpe_cmp (a,b)
#define CMP_D(a,b) dpe_cmp_d (a,b) 
#define SGN(a) dpe_cmp_d (a,0)
#define RND(a,b) dpe_round (a,b)
#define GET_Z_EXP(a,b) dpe_get_z_exp(a,b) 
#define SWAP(a,b) dpe_swap (a,b)
#define ZERO_P(a) dpe_zero_p (a)

#define SET_NAN(a) dpe_set_d (a, 0.0/0.0)
#define IS_NAN(a) (DPE_MANT(a)!=DPE_MANT(a))
#define SQRT(a,b) dpe_sqrt(a, b)



#else
#ifdef USE_LONGDOUBLE


typedef long double long_double_t[1];

#define FP_NR long_double_t
#define INIT(a)  {}
#define CLEAR(a) {}
#define OUT(a) printf ("%LE ", a[0]) 
#define OUT2(a) fprintf (stderr, "%LE ", a[0])
#define SET(a,b) a[0] = b[0]
#define SET_D(a,b) a[0] = b
#define GET_D(b) (double) b[0]
#define SET_UI(a,b) a[0] = (long double) b

#define MUL(a,b,c) a[0] = b[0] * c[0]
#define ADD(a,b,c) a[0] = b[0] + c[0]
#define SUB(a,b,c) a[0] = b[0] - c[0]
#define DIV(a,b,c) a[0] = b[0] / c[0] 
#define DIV_2UI(a,b,c) a[0]= ldexp (b[0], -c)
#define MUL_2UI(a,b,c) a[0]= ldexp (b[0], c)

#define ABS(a,b) a[0] = (b[0]>=0.0? b[0]: -b[0])
#define CMP(a,b) a[0] <= b[0] ? -1 : 1
#define CMP_D(a,b) a[0] <= b ? -1 : 1
#define SGN(a) (a[0]>0.0? 1: (a[0]==0.0? 0 : -1))
#define RND(a,b) a[0] = rintl(b[0])

mp_exp_t long_double_get_z_exp (mpz_t, long double*);
int long_double_get_z (mpz_t, long double*);
void long_double_set_z (long double*, mpz_t);

#define SET_Z(a,b) long_double_set_z (a, b)
#define GET_SI(a,b) a = (signed long int) b[0]
#define GET_Z_EXP(a,b) long_double_get_z_exp (a,b)
#define GET_Z(a,b) long_double_get_z (a,b)

#define SET_NAN(a)  a[0] = (long double) (0.0/0.0) 
#define IS_NAN(a) (a[0]!=a[0])
#define SQRT(a,b) a[0] = (long double) sqrt(b[0])

#endif
#endif
#endif
#endif
